package jp.co.asterisk.asreader.a24d.demoapp;

import android.app.Activity;
import android.app.Application;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import android.util.Log;

import java.util.ArrayList;
import java.util.Map;

import jp.co.asterisk.asreader.a24d.demoapp.databinding.ActivityMainBinding;
import jp.co.asterisk.asreader.a24d.demoapp.fragments.BaseFragment;
import jp.co.asterisk.asreader.a24d.demoapp.fragments.InfoFragment;
import jp.co.asterisk.asreader.a24d.demoapp.fragments.MainFragment;
import jp.co.asterisk.asreader.a24d.demoapp.fragments.SettingFragment;
import jp.co.asterisk.asreader.a24d.demoapp.util.symbology.SymbologyPreferencesHelper;
import jp.co.asterisk.asreader.a24d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a24d.demoapp.util.BarcodeData;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;
import jp.co.asterisk.asreader.communicationmanager.connection.ConnectionType;
import jp.co.asterisk.asreader.devicemanager.*;
import jp.co.asterisk.asreader.utility.AsReaderError;

public class MainActivity extends AppCompatActivity implements DeviceManagerCallback {

    private static final String TAG = "【A24DDemoApp】" + MainActivity.class.getSimpleName();
    private static final String TAG_TEST = "【A24DDemoAppTestActivity】" + MainActivity.class.getSimpleName();

    private ActivityMainBinding binding;
    private ArrayList<BarcodeData> mDataList;
    public enum AppStatus {
        INTO_BACKGROUND,
        BACKGROUND,
        FOREGROUND
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        AsReader.getInstance().setLog(true);

        getApplication().registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks());

        SharedPreferencesHelper.getInstance().initialize(getApplicationContext());
        SymbologyPreferencesHelper.getInstance().initialize(getApplicationContext());
        mDataList = new ArrayList<>();

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        binding.toolbar.setTitle("");
        setSupportActionBar(binding.toolbar);

        AsReader.getInstance().initialize(getApplicationContext(), ConnectionType.USB);
        AsReader.getInstance().getDeviceManager().setCallback(this);
        AsReader.getInstance().getDeviceManager().setPackageName(getPackageName());

        binding.backButton.setOnClickListener(v-> {
            Fragment currentFragment = getCurrentFragment();
            if( currentFragment!=null ) {
                if (currentFragment instanceof MainFragment) {
                    ((BaseFragment) currentFragment).onLeftButtonClick();
                } else {
                    NavHostFragment.findNavController(currentFragment).popBackStack();
                }
            }
        });

        binding.rightButton.setOnClickListener(v-> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment)currentFragment).onRightButtonClick();
            }
        });
    }

    public void setTitle( @NonNull String title) {
        if(binding!=null) {
            binding.titleTextView.setText(title);
        }
    }

    public void setBackButtonVisibility(int visibility) {
        if(binding!=null){
            binding.backButton.setVisibility(visibility);
        }
    }

    public void setBackButtonEnabled(boolean enabled) {
        if(binding!=null) {
            binding.backButton.setEnabled(enabled);
        }
    }

    public void setBackButtonText(String text) {
        if(binding!=null) {
            binding.backButton.setText(text);
        }
    }

    public void setRightButtonVisibility(int visibility) {
        if(binding!=null) {
            binding.rightButton.setVisibility(visibility);
        }
    }

    public void setRightButtonEnabled(boolean enabled) {
        if(binding!=null) {
            binding.rightButton.setEnabled(enabled);
        }
    }

    public void setRightButtonText(String text) {
        if(binding!=null) {
            binding.rightButton.setText(text);
        }
    }

    public ArrayList<BarcodeData> getTagDataList() {
        return mDataList;
    }

    private Fragment getCurrentFragment() {
        Fragment navHostFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
        return (navHostFragment != null) ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;
    }

    // *******************************************************
    //  DeviceManagerCallback
    // *******************************************************
    @Override
    public void onConnect(boolean isConnect) {
        Log.d(TAG, "onConnect : " + isConnect);

        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment) currentFragment).onConnect(isConnect);
            }
        });
    }

    @Override
    public void onBatteryStateReceived(int battery) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof MainFragment) {
                ((MainFragment) currentFragment).onBatteryStateReceived(battery);
            }
        });
    }

    @Override
    public void onTriggerEventReceived(DeviceConst.DeviceTriggerEvent event) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof MainFragment) {
                ((MainFragment) currentFragment).onTriggerEventReceived(event);
            }
        });
    }

    @Override
    public void onSetReaderSettingSuccess() {
        Log.d(TAG, "onSetReaderSettingSuccess");
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onSetReaderSettingSuccess();
            }
        });
    }

    @Override
    public void onReaderSettingReceived(Map<String, Boolean> settingParamMap) {
        Log.d(TAG, "onGetReaderSettingSuccess");
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).getReaderSettings(settingParamMap);
            }
        });
    }

    @Override
    public void onReaderInfoReceived(Map<String, String> info) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof InfoFragment) {
                String model = info.get(DeviceConst.KEY_INFO_MODEL_NAME);
                String hwVersion = info.get(DeviceConst.KEY_INFO_HARDWARE_VERSION);
                String fwVersion = info.get(DeviceConst.KEY_INFO_FIRMWARE_VERSION);

                ((InfoFragment) currentFragment).setReaderInfo(model, hwVersion, fwVersion);
            }
        });
    }

    @Override
    public void onSleepSettingReceived(Map<String, String> paramMap) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onSleepSettingReceived(paramMap);
            }
        });
    }

    @Override
    public void onSetChargeControlSuccess() {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).setChargeControlSuccess();
            }
        });
    }

    @Override
    public void onHIDModeReceived(DeviceConst.HIDType hidType) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onHIDModeReceived(hidType);
            }
            if (currentFragment instanceof MainFragment) {
                ((MainFragment) currentFragment).onHIDModeReceived(hidType);
            }
        });
    }

    @Override
    public void onDeviceManagerError(AsReaderError.ErrorCode code) {
        Log.d("MainFragment", "onDeviceManagerError: " + code.getMessage());

        if( code== AsReaderError.ErrorCode.ALREADY_POWER_ON ) {
            return;
        }

        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment) currentFragment).onDeviceError(code.getMessage());
            }
        });
    }

    // *******************************************************
    //  ActivityCallback
    // *******************************************************
    class ActivityLifecycleCallbacks implements Application.ActivityLifecycleCallbacks {

        @Override
        public void onActivityCreated(@NonNull Activity activity, @Nullable Bundle savedInstanceState) {
            Log.d(TAG_TEST, "onActivityCreated");
        }

        @Override
        public void onActivityStarted(@NonNull Activity activity) {
            Log.d(TAG_TEST, "onActivityStarted");
            runOnUiThread(() -> {
                Fragment currentFragment = getCurrentFragment();
                if (currentFragment instanceof BaseFragment) {
                    ((BaseFragment) currentFragment).receivedAppStatus(AppStatus.FOREGROUND);
                }
            });
        }

        @Override
        public void onActivityResumed(@NonNull Activity activity) {
            Log.d(TAG_TEST, "onActivityResumed");
        }

        @Override
        public void onActivityPaused(@NonNull Activity activity) {
            Log.d(TAG_TEST, "onActivityPaused");
            runOnUiThread(() -> {
                Fragment currentFragment = getCurrentFragment();
                if (currentFragment instanceof BaseFragment) {
                    ((BaseFragment) currentFragment).receivedAppStatus(AppStatus.INTO_BACKGROUND);
                }
            });
        }

        @Override
        public void onActivityStopped(@NonNull Activity activity) {
            Log.d(TAG_TEST, "onActivityStopped");
            runOnUiThread(() -> {

                Fragment currentFragment = getCurrentFragment();
                if (currentFragment instanceof BaseFragment) {
                    ((BaseFragment) currentFragment).receivedAppStatus(AppStatus.BACKGROUND);
                }
            });
        }

        @Override
        public void onActivitySaveInstanceState(@NonNull Activity activity, @NonNull Bundle outState) {
            Log.d(TAG_TEST, "onActivitySaveInstanceState");
        }

        @Override
        public void onActivityDestroyed(@NonNull Activity activity) {
            Log.d(TAG_TEST, "onActivityDestroyed");
        }
    }
}