package jp.co.asterisk.asreader.a24d.demoapp.fragments;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.navigation.fragment.NavHostFragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import jp.co.asterisk.asreader.a24d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.a24d.demoapp.databinding.FragmentInfoBinding;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;

public class InfoFragment extends BaseFragment {

    private final String TAG = InfoFragment.class.getSimpleName();
    private FragmentInfoBinding binding;

    public InfoFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        binding = FragmentInfoBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Log.d( TAG, "onViewCreated");
        setTitle(getString(R.string.information));
        setBackButtonVisibility(View.VISIBLE);
        setRightButtonVisibility(View.VISIBLE);
        setRightButtonEnabled(true);
        setRightButtonText(getString(R.string.refresh));
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d( TAG, "onResume");
        requestReaderInformation();
    }

    private void requestReaderInformation() {
        binding.infoTextView.append("SDK: "+ AsReader.getInstance().getVersion());
        AsReader.getInstance().getDeviceManager().getReaderInfo();
    }

    @Override
    public void onRightButtonClick() {
        binding.infoTextView.setText("");
        requestReaderInformation();
    }

    @Override
    public void onConnect(boolean isConnect) {
        Log.d( TAG, "onConnect");
        if(!isConnect){
            NavHostFragment.findNavController(InfoFragment.this).navigate(R.id.action_InfoFragment_to_MainFragment);
        }
    }

    @Override
    public void receivedAppStatus(MainActivity.AppStatus status) {
        if( status==MainActivity.AppStatus.BACKGROUND || status==MainActivity.AppStatus.INTO_BACKGROUND) {
            NavHostFragment.findNavController(InfoFragment.this).navigate(R.id.action_InfoFragment_to_MainFragment);
        }
    }

    public void setReaderInfo(String model, String hwVersion, String fwVersion) {
        binding.infoTextView.append("\n Model: " + model + "\n HW ver. " + hwVersion + "\n FW ver. " + fwVersion);
    }


}