package jp.co.asterisk.asreader.a24d.demoapp.fragments;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ListView;

import androidx.annotation.NonNull;
import androidx.navigation.fragment.NavHostFragment;

import java.util.ArrayList;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import jp.co.asterisk.asreader.a24d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.a24d.demoapp.databinding.FragmentMainBinding;
import jp.co.asterisk.asreader.a24d.demoapp.util.PopUpDialog;
import jp.co.asterisk.asreader.a24d.demoapp.util.SettingConst;
import jp.co.asterisk.asreader.a24d.demoapp.util.BarcodeDataListAdapter;
import jp.co.asterisk.asreader.a24d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a24d.demoapp.util.BarcodeData;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;

import jp.co.asterisk.asreader.barcodemanager.*;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeConst;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeManagerError;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeResult;
import jp.co.asterisk.asreader.devicemanager.DeviceConst;
import jp.co.asterisk.asreader.devicemanager.DeviceManager;

public class MainFragment extends BaseFragment implements BarcodeManagerCallback {

    private static final String TAG = "【A24DDemoApp】" + MainFragment.class.getSimpleName();
    private static final String TAG_TEST = "【A24DDemoAppTest】" + MainFragment.class.getSimpleName();

    private FragmentMainBinding binding;

    private ArrayList<BarcodeData> mDataList;
    private BarcodeDataListAdapter mDataListAdapter;
    private String mBarcodeHexString = "";
    final Handler handler = new Handler(Looper.getMainLooper());
    private boolean handlerRun = false;
    private boolean isViewCreated = false;
    private DeviceConst.HIDType hidType = null;

    private Timer timer;
    private long elapsed = 0;
    private final long TIMER_INTERVAL = 1000;
    private final long TIMER_TIME_OUT = 1000 * 60 * 5 ;

    private SoundPool mSoundPool = null;
    private int mSoundId;

    //-----------------------------------------------------------------------------
    // Lifecycle
    //-----------------------------------------------------------------------------
    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {
        // Inflate the layout for this fragment
        binding = FragmentMainBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {

        super.onViewCreated(view, savedInstanceState);

        Log.d(TAG, "onViewCreated");

        // Connection Type set USB
        String versionText = "App Version : " + getAppVersion();
        binding.versionTextview.setText(versionText);

        binding.readButton.setOnClickListener(v -> {
            if (AsReader.getInstance().getDeviceManager().isConnected() ) {
                AsReader.getInstance().getBarcodeManager().startDecode();
                startTimer();
            }
        });

        binding.clearButton.setOnClickListener(v -> {
                mDataList.clear();
                mDataListAdapter.notifyDataSetChanged();
                binding.tagCountTextView.setText("0 Barcode");
        });

        binding.stopButton.setOnClickListener(v -> {
            if ( AsReader.getInstance().getDeviceManager().isConnected() ) {
                AsReader.getInstance().getBarcodeManager().stopDecode();
            }
        });

        binding.deviceButton.setOnClickListener(v -> {
            StringBuilder information = new StringBuilder();
            UsbManager usbManager = (UsbManager) this.getContext().getSystemService(Context.USB_SERVICE);

            information.append("Device count : "+usbManager.getDeviceList().size());
            if ( !usbManager.getDeviceList().isEmpty() ) {
                for (UsbDevice object : usbManager.getDeviceList().values()) {
                    information.append("\nPermission : " +usbManager.hasPermission(object)
                            +"\nVID : "+object.getVendorId());
                }
            }

            PopUpDialog popUpDialog = new PopUpDialog(getContext(),information.toString());
            popUpDialog.show();
        });

        mDataList = ((MainActivity) requireActivity()).getTagDataList();
        if( mDataList!=null ) {
            mDataListAdapter = new BarcodeDataListAdapter(requireContext());
            mDataListAdapter.setDataList(mDataList);
            ListView dataListView = view.findViewById(R.id.tag_list_view);
            dataListView.setAdapter(mDataListAdapter);
            dataListView.setFocusable(false);
            dataListView.setFocusableInTouchMode(false);
            dataListView.setOnItemClickListener((parent, view1, position, id) -> {
                AsReader.getInstance().getBarcodeManager().stopDecode();

                Bundle args = new Bundle();
                if(mDataList==null) { return; }
                // Create text
                String displayText;
                if( mDataList.get(position).getCharacterCodeID()!=null && mDataList.get(position).getCharacterCodeID().length()>0 ) {
                    displayText = "【 CodeID 】 " + mDataList.get(position).getCharacterCodeID() + "\n";
                } else {
                    displayText = "【 CodeID 】 " + "Could not get."+ "\n";
                }

                if( mDataList.get(position).getPrefix()!=null && mDataList.get(position).getPrefix().length()>0 ) {
                    displayText = displayText + "【 Prefix 】 " + mDataList.get(position).getPrefix() + "\n";
                }

                if( mDataList.get(position).getSuffix1()!=null && mDataList.get(position).getSuffix1().length()>0 ) {
                    displayText =  displayText + "【 Suffix1 】 " + mDataList.get(position).getSuffix1() + "\n";
                }
                if( mDataList.get(position).getSuffix2()!=null && mDataList.get(position).getSuffix2().length()>0 ) {
                    displayText = displayText + "【 Suffix2 】 " + mDataList.get(position).getSuffix2() + "\n";
                }
                displayText = displayText + "\n";
                displayText = displayText + "【 Barcode(TEXT) 】" + "\n" + mDataList.get(position).getBarcode() + "\n" + "\n";
                displayText = displayText + "【 Barcode(HEX) 】" + "\n" + mDataList.get(position).getHexData() + "\n";

                args.putString("DisplayText", displayText);

                NavHostFragment.findNavController(MainFragment.this)
                        .navigate(R.id.action_MainFragment_to_BarcodeAccessFragment, args);
            });

            String countText = mDataListAdapter.getCount() + " Barcode";
            binding.tagCountTextView.setText(countText);
        }

        isViewCreated = true;
    }

    @Override
    public void onStart(){
        super.onStart();
        Log.d(TAG, "onStart");

        setTitle(getString(R.string.app_name));
        setRightButtonText(getString(R.string.more));

        setBackButtonVisibility(View.INVISIBLE);
        setRightButtonVisibility(View.VISIBLE);

        if( AsReader.getInstance().getDeviceManager().isConnected() ){
            binding.deviceStatusTextView.setText(R.string.connect_status_connected);
            setRightButtonEnabled(true);
        } else {
            binding.deviceStatusTextView.setText(R.string.connect_status_disconnected);
            setRightButtonEnabled(false);
        }
        AsReader.getInstance().getBarcodeManager().setCallback(this);

        boolean launchMode = SharedPreferencesHelper.getInstance().getAppAutoLaunch();
        if( launchMode ) {
            setLeftButtonText(" LaunchMode ON");
        } else {
            setLeftButtonText(" LaunchMode OFF");
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume");
        connectProcess();
        setSoundPool();
    }

    @Override
    public void onPause() {
        super.onPause();
        Log.d(TAG, "onPause");
        hideProgressDialog();
        AsReader.getInstance().getBarcodeManager().stopDecode();
        stopTimer();
        handlerRun = false;
        releaseSoundPool();
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
    }

    //-----------------------------------------------------------------------------
    // BaseFragment Override method
    //-----------------------------------------------------------------------------
    @Override
    public void onRightButtonClick() {
        NavHostFragment.findNavController(MainFragment.this).navigate(R.id.action_MainFragment_to_MoreFragment);
    }

    @Override
    public void onLeftButtonClick() {
//        createRebootAlertDialog().show();
    }

    @Override
    public void onConnect(boolean isConnect) {
        if ( isViewCreated ){
            if (isConnect) {
                binding.deviceStatusTextView.setText(R.string.connect_status_connected);
            } else {
                binding.deviceStatusTextView.setText(R.string.connect_status_disconnected);
                handlerRun = false;
                hidType = null;
                showCurrentMode();
            }
            connectProcess();
        }
    }

    @Override
    public void onDeviceError(String message) {
        Log.d(TAG, "Error message : " + message);
        hideProgressDialog();
    }

    //-----------------------------------------------------------------------------
    //  Public method
    //-----------------------------------------------------------------------------
    public void onBatteryStateReceived(int battery) {
        SettingConst.batteryValue = battery;
        binding.batteryTextView.setText(battery + "%");
    }

    public void onTriggerEventReceived(DeviceConst.DeviceTriggerEvent event) {
        Log.i(TAG, "Event = " + event);
//        switch (event){
//            case PUSH_LEFT:
//            case PUSH_RIGHT:
//                AsReader.getInstance().getBarcodeManager().startDecode();
//                break;
//
//            case PUSH_NONE:
//                AsReader.getInstance().getBarcodeManager().stopDecode();
//                break;
//        }
    }

    public void onHIDModeReceived(DeviceConst.HIDType hidType) {
        this.hidType = hidType;
        showCurrentMode();
    }

    //-----------------------------------------------------------------------------
    //  Private method
    //-----------------------------------------------------------------------------
    private synchronized boolean updateBarcodeDataList(String barcode, Map<String, String>parameter) {

        boolean itemExists = false;

        String codeIDSymbol = null;
        String codeIDAIM = null;
        String prefix = null;
        String suffix1 = null;
        String suffix2 = null;

        if( parameter!=null ) {
            codeIDSymbol = parameter.get(BarcodeConst.KEY_PARAMETER_CODE_ID_SYMBOL);
            codeIDAIM = parameter.get(BarcodeConst.KEY_PARAMETER_CODE_ID_AIM);
            prefix = parameter.get(BarcodeConst.KEY_PARAMETER_PREFIX);
            suffix1 = parameter.get(BarcodeConst.KEY_PARAMETER_SUFFIX1);
            suffix2 = parameter.get(BarcodeConst.KEY_PARAMETER_SUFFIX2);
        }

        Activity activity = getActivity();
        if (activity != null) {
            Log.i("foo", activity.getLocalClassName());
        }

        for( int i=0; i<mDataList.size(); i++ ) {

            if( mDataList.get(i)==null ) { continue; }

            itemExists = mDataList.get(i).getBarcode().contentEquals(barcode);
            if ( itemExists ) {
                mDataList.get(i).setCount( mDataList.get(i).getCount()+1 );

                if( codeIDSymbol!=null ) {
                    mDataList.get(i).setCharacterCodeID(codeIDSymbol);
                } else if( codeIDAIM!=null ) {
                    mDataList.get(i).setCharacterCodeID(codeIDAIM);
                } else {
                    mDataList.get(i).setCharacterCodeID(null);
                }

                mDataList.get(i).setPrefix(prefix);
                mDataList.get(i).setSuffix1(suffix1);
                mDataList.get(i).setSuffix2(suffix2);
                break;
            }
        }

        if (!itemExists) {
            BarcodeData newData = new BarcodeData(barcode);

            if( codeIDSymbol!=null ) {
                newData.setCharacterCodeID(codeIDSymbol);
            } else if( codeIDAIM!=null ) {
                newData.setCharacterCodeID(codeIDAIM);
            } else {
                newData.setCharacterCodeID(null);
            }

            newData.setPrefix(prefix);
            newData.setSuffix1(suffix1);
            newData.setSuffix2(suffix2);
            newData.setHexData(mBarcodeHexString);
            mDataList.add(newData);
        }
        return true;
    }

    private void connectProcess() {
        setDeviceDefault();

        if( handlerRun ) { return; }

        handler.post(() -> {
            // UIスレッド
            boolean isConnected = AsReader.getInstance().getDeviceManager().isConnected();

            if ( isConnected && isViewCreated ) {
                setRightButtonEnabled(true);
                DeviceManager.getInstance().getHIDMode();
            } else {
                setRightButtonEnabled(false);
                binding.batteryTextView.setText("Unknown");
            }
        });
    }

    private void setDeviceDefault() {
        boolean beep = SharedPreferencesHelper.getInstance().getBeep();
        boolean vib = SharedPreferencesHelper.getInstance().getVibration();
        boolean aimer = SharedPreferencesHelper.getInstance().getAimer();
        boolean led = SharedPreferencesHelper.getInstance().getLed();
        boolean powerBeep = SharedPreferencesHelper.getInstance().getPowerBeep();
        AsReader.getInstance().getDeviceManager().setReaderSettings(beep, vib, aimer, led, powerBeep);
    }

    // Keep screen on if true
    private void keepScreenOnMode(Activity activity, boolean keepScreenOn){
        if (getActivity()!=null){
            getActivity().runOnUiThread(() -> {
                if(keepScreenOn){
                    activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                } else {
                    activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                }
            });
        }
    }

    private void startTimer(){
        keepScreenOnMode(getActivity(), true);

        if(timer != null) {
            return;
        }
        TimerTask task = new TimerTask(){
            @Override
            public void run() {
                elapsed += TIMER_INTERVAL;
                if(elapsed >= TIMER_TIME_OUT){
                    stopTimer();
                }
            }
        };
        timer = new Timer();
        timer.scheduleAtFixedRate(task, TIMER_INTERVAL, TIMER_INTERVAL);
    }

    private void stopTimer(){
        keepScreenOnMode(getActivity(), false);

        if(timer != null){
            timer.cancel();
            timer.purge();
            timer = null;
            elapsed = 0;
        }
    }

    private String getAppVersion() {
        String versionName;
        String packageName = requireActivity().getPackageName();

        // Get Application Version
        try {
            versionName = requireActivity().getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA).versionName;
        } catch (PackageManager.NameNotFoundException e) {
            versionName = "Unknown";
        }
        return versionName;
    }

    private void setSoundPool(){
        SettingConst.ApplicationBeep applicationBeep = SharedPreferencesHelper.getInstance().getApplicationBeep();

        if( applicationBeep!=SettingConst.ApplicationBeep.NONE ) {
            mSoundPool = new SoundPool.Builder().setMaxStreams(1).build();
            // load sound pool
            mSoundId = mSoundPool.load(requireActivity().getApplicationContext(), applicationBeep.getSoundID(), 1);
        }
    }

    private void releaseSoundPool(){
        if(mSoundPool!=null){
            mSoundPool.release();
            mSoundPool = null;
        }
    }

    private void playApplicationBeepSound() {
        SettingConst.ApplicationBeep applicationBeep = SharedPreferencesHelper.getInstance().getApplicationBeep();
        if( applicationBeep==SettingConst.ApplicationBeep.NONE || mSoundPool==null ) {
            return;
        }

        try {
            mSoundPool.play(mSoundId, 1f, 1f, 1, 0, 1f);
        } catch(Exception e) {
            Log.w(TAG, "SoundPool exception : " + e.getMessage());
        }
    }

    private void showCurrentMode(){
        if(hidType == null){
            binding.modeTextview.setText("");
        } else {
            binding.modeTextview.setText( hidType == DeviceConst.HIDType.HID_ON ? "HID" : "CDC");
        }
    }

    //-----------------------------------------------------------------------------
    //  BarcodeManagerCallback
    //-----------------------------------------------------------------------------
    @Override
    public void onBarcodeManagerStandByReady() {
        Log.d(TAG_TEST, "onBarcodeManagerStandByReady");
    }

    @Override
    public void onReceivedBarcodeDecodeData(String decodeData, Map<String, String> parameter) {
        requireActivity().runOnUiThread(() -> {

            Log.d(TAG_TEST, "onReceivedData : " + "Barcode = " + decodeData + "  CodeID = " + parameter);

            playApplicationBeepSound();

            if( updateBarcodeDataList(decodeData, parameter) ) {
                binding.tagCountTextView.setText(mDataListAdapter.getCount() + " Barcode");
                mDataListAdapter.notifyDataSetChanged();
            }
        });
        // Reset timer
        elapsed = 0;
    }

    @Override
    public void onReceivedBarcodeDecodeData(byte[] decodeData) {
        mBarcodeHexString = "";
        for (byte decodeDatum : decodeData) {
            mBarcodeHexString = mBarcodeHexString + String.format("%02X ", decodeDatum);
        }
        Log.d(TAG, "Barcode byte data = " + mBarcodeHexString );
    }

    @Override
    public void onBarcodeManagerError(BarcodeManagerError error) {
        Log.e(TAG, "onBarcodeManagerError: " + error.getMessage());
    }

    @Override
    public void onReceivedResponse(BarcodeConst.BarcodeCommandAction type, BarcodeResult result) {
        Log.i(TAG_TEST, "onReceivedResponse : type = " + type + " Result = " + result.getMessage());
    }
}