package jp.co.asterisk.asreader.a24d.demoapp.fragments;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.navigation.fragment.NavHostFragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import jp.co.asterisk.asreader.a24d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.a24d.demoapp.databinding.FragmentMoreBinding;
import jp.co.asterisk.asreader.a24d.demoapp.util.SettingAdapter;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;
import jp.co.asterisk.asreader.barcodemanager.*;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeManagerError;

public class MoreFragment extends BaseFragment implements BarcodeManagerCallback {

    private final String LIST_TITLE_SETTINGS = "Reader&Barcode Settings";
    private final String LIST_TITLE_INFORMATION = "Reader Information";
    private static final String TAG_TEST_LIFECYCLE = "【A24DDemoAppTestActivity】" + MainActivity.class.getSimpleName();

    private FragmentMoreBinding binding;
    //-----------------------------------------------------------------------------
    //  Constructor
    //-----------------------------------------------------------------------------
    public MoreFragment() {
        // Required empty public constructor
    }
    

    //-----------------------------------------------------------------------------
    //  Lifecycle
    //-----------------------------------------------------------------------------
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onStart(){
        super.onStart();
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        binding = FragmentMoreBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setTitle(getString(R.string.more));
        setBackButtonVisibility(View.VISIBLE);
        setLeftButtonEnabled(true);
        setLeftButtonText("BACK");

        setRightButtonVisibility(View.INVISIBLE);


        SettingAdapter settingAdapter = new SettingAdapter(getContext());
        settingAdapter.add(LIST_TITLE_SETTINGS);
        settingAdapter.add(LIST_TITLE_INFORMATION);

        binding.optionListView.setAdapter(settingAdapter);
        binding.optionListView.setOnItemClickListener((parent, view1, position, id) -> {
            if (position == 0) {
                NavHostFragment.findNavController(MoreFragment.this).navigate(R.id.action_MoreFragment_to_SettingFragment);
            } else if (position == 1) {
                NavHostFragment.findNavController(MoreFragment.this).navigate(R.id.action_MoreFragment_to_InfoFragment);
            }
        });
    }

    //-----------------------------------------------------------------------------
    //  BaseFragment Override method
    //-----------------------------------------------------------------------------
    @Override
    public void onConnect(boolean isConnect) {
        Log.d("MoreFragment", "onConnect");
        Log.d(TAG_TEST_LIFECYCLE, "onConnect (MoreFragment): " + isConnect);
        if (!isConnect) {
            NavHostFragment.findNavController(MoreFragment.this).navigate(R.id.action_MoreFragment_to_MainFragment);
        }
    }

    @Override
    public void receivedAppStatus(MainActivity.AppStatus status) {
        if( status==MainActivity.AppStatus.BACKGROUND || status==MainActivity.AppStatus.INTO_BACKGROUND) {
            NavHostFragment.findNavController(MoreFragment.this).navigate(R.id.action_MoreFragment_to_MainFragment);
        }
    }

    //-----------------------------------------------------------------------------
    //  BarcodeManagerCallback
    //-----------------------------------------------------------------------------
    @Override
    public void onBarcodeManagerError(BarcodeManagerError error) {
        Log.d("MoreFragment", "onBarcodeManagerError");
        requireActivity().runOnUiThread(() -> showToast(error.getMessage()));
    }
}