package jp.co.asterisk.asreader.a24d.demoapp.fragments;

import android.app.AlertDialog;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.navigation.fragment.NavHostFragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;

import jp.co.asterisk.asreader.a24d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.a24d.demoapp.databinding.FragmentSettingBinding;
import jp.co.asterisk.asreader.a24d.demoapp.util.SettingConst;
import jp.co.asterisk.asreader.a24d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;
import jp.co.asterisk.asreader.barcodemanager.BarcodeManager;
import jp.co.asterisk.asreader.barcodemanager.BarcodeManagerCallback;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeConst;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeManagerError;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeResult;
import jp.co.asterisk.asreader.barcodemanager.define.OCR;
import jp.co.asterisk.asreader.barcodemanager.model.OCRInfoModel;
import jp.co.asterisk.asreader.devicemanager.DeviceConst;
import jp.co.asterisk.asreader.devicemanager.DeviceManager;

public class SettingFragment extends BaseFragment implements BarcodeManagerCallback {

    private static final String TAG = "【AsReader DemoApp】" + SettingFragment.class.getSimpleName();
    private static final String TAG_TEST = "【SettingFragmentTest】" + SettingFragment.class.getSimpleName();
    private static final String TAG_CHARACTER_TEST = "【SettingFragmentTest CharacterStatus】" + SettingFragment.class.getSimpleName();

    private enum GetProcessStep {
        UNPROCESSED,
        SLEEP_TIME,
        HID_MODE,
        CHARACTER_STATUS,
        NOTIS_STATUS,
        FINISH,
    }

    private FragmentSettingBinding binding;
    private SettingConst.IntoSleepMode mIntoSleepMode = SettingConst.IntoSleepMode.MINUTE_5;
    private SettingConst.CharacterStatus mCharacterStatus = SettingConst.CharacterStatus.NONE;
    private SettingConst.ApplicationBeep mApplicationBeepSound = SettingConst.ApplicationBeep.BEEP1;
    private ArrayList<OCRInfoModel> mOCRInfo = SettingConst.getFalseOCRArray();
    private MediaPlayer mMediaPlayer = new MediaPlayer();
    private GetProcessStep mProcessStep = GetProcessStep.UNPROCESSED;

    //-----------------------------------------------------------------------------
    //  Constructor
    //-----------------------------------------------------------------------------
    public SettingFragment() {
        // Required empty public constructor
    }

    //-----------------------------------------------------------------------------
    //  Lifecycle
    //-----------------------------------------------------------------------------
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        binding = FragmentSettingBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onStart(){
        super.onStart();
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override public void onResume(){
        super.onResume();
    }

    @Override
    public void onPause(){
        super.onPause();
        mProcessStep = GetProcessStep.UNPROCESSED;
        hideProgressDialog();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setTitle(getString(R.string.setting));
        setBackButtonVisibility(View.VISIBLE);
        setRightButtonVisibility(View.INVISIBLE);

        binding.beepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getBeep());
        binding.beepToggleButton.setOnClickListener(v -> {
            showProgressDialog();
            changeSetting();
        });

        binding.vibrationToggleButton.setChecked(SharedPreferencesHelper.getInstance().getVibration());
        binding.vibrationToggleButton.setOnClickListener(v -> {
            showProgressDialog();
            changeSetting();
        });

        binding.ledToggleButton.setChecked(SharedPreferencesHelper.getInstance().getLed());
        binding.ledToggleButton.setOnClickListener(v -> {
            showProgressDialog();
            changeSetting();
        });

        binding.aimerToggleButton.setChecked(SharedPreferencesHelper.getInstance().getAimer());
        binding.aimerToggleButton.setOnClickListener(v -> {
            showProgressDialog();
            changeSetting();
        });

        binding.powerBeepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getPowerBeep());
        binding.powerBeepToggleButton.setOnClickListener(v -> {
            showProgressDialog();
            changeSetting();
        });

        binding.sleepBeepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getSleepBeep());
        binding.sleepBeepToggleButton.setOnClickListener(V -> {
            changeSleepSetting();
        });

        switch ( AsReader.getInstance().getBarcodeManager().getSystemBeep() ) {
            case NONE:
                binding.ssiBeepToggleButton.setChecked(false);
                break;
            default:
                binding.ssiBeepToggleButton.setChecked(true);
        }
        binding.ssiBeepToggleButton.setOnClickListener(v -> {
            if( binding.ssiBeepToggleButton.isChecked() ) {
                BarcodeManager.getInstance().setSystemBeep(BarcodeConst.SystemBeepSoundType.TYPE2);
            } else {
                BarcodeManager.getInstance().setSystemBeep(BarcodeConst.SystemBeepSoundType.NONE);
            }
        });

        binding.autoLaunchModeToggleButton.setChecked(SharedPreferencesHelper.getInstance().getAppAutoLaunch());
        binding.autoLaunchModeToggleButton.setOnClickListener(v -> {
            boolean enable = binding.autoLaunchModeToggleButton.isChecked();
            AsReader.getInstance().enableAutoLaunchApp(enable, "MainActivityAlias");
            SharedPreferencesHelper.getInstance().setAppAutoLaunch(enable);
        });

        binding.codeIdSelectButton.setOnClickListener( v-> createCodeIdAlert().show());
        binding.sleepTimeSelectButton.setOnClickListener( v-> createSleepAlert().show());
        binding.applicationBeepSelectButton.setOnClickListener( v-> createApplicationBeepAlert().show());
        binding.ocrSelectButton.setOnClickListener( v-> createOCRAlert().show());
        ArrayList<OCR> ocr = new ArrayList<>();
        Collections.addAll(ocr, OCR.values());
        AsReader.getInstance().getBarcodeManager().getOCRSettings(ocr);


        binding.setHidToggleButton.setChecked(SharedPreferencesHelper.getInstance().getHidMode());
        binding.setHidToggleButton.setOnClickListener( v-> {
            if(SettingConst.batteryValue <= 20){
                showToast("Battery is too low");
                return;
            }
            boolean enable = binding.setHidToggleButton.isChecked();
            AsReader.getInstance().getDeviceManager().setHIDMode(enable);
            // change auto lauch mode off
            AsReader.getInstance().enableAutoLaunchApp(false, "MainActivityAlias");
            SharedPreferencesHelper.getInstance().setAppAutoLaunch(false);
            // change HID mode
            SharedPreferencesHelper.getInstance().setHidMode(enable);
            // finish app
            getActivity().finish();
        });

        binding.setBackgroundReadToggleButton.setChecked(AsReader.getInstance().getDeviceManager().getBackgroundRead());
        binding.setBackgroundReadToggleButton.setOnClickListener( v-> {
            /*boolean enable = binding.setBackgroundReadToggleButton.isChecked();
            AsReader.getInstance().getDeviceManager().setBackgroundRead(enable);
            SharedPreferencesHelper.getInstance().setBackgroundRead(enable);
            showToast(enable ? "Background Read On" : "Background Read Off");*/
        });

        switch (AsReader.getInstance().getBarcodeManager().getDecodeMode()){
            case DECODE_MODE_SINGLE:
                binding.autoScanModeToggleButton.setChecked(false);
                break;
            case DECODE_MODE_CONTINUOUS:
                binding.autoScanModeToggleButton.setChecked(true);
                break;
        }
        binding.autoScanModeToggleButton.setOnClickListener(v ->{
            if( !binding.autoScanModeToggleButton.isChecked() ){
                AsReader.getInstance().getBarcodeManager().setDecodeMode(BarcodeConst.BarcodeSettings.DECODE_MODE_SINGLE);
            } else {
                AsReader.getInstance().getBarcodeManager().setDecodeMode(BarcodeConst.BarcodeSettings.DECODE_MODE_CONTINUOUS);
            }
        });

        switch (AsReader.getInstance().getDeviceManager().getTriggerMode()){
            case MANUAL:
                binding.triggerModeToggleButton.setChecked(false);
                break;
            case AUTO:
                binding.triggerModeToggleButton.setChecked(true);
                break;
        }
        binding.triggerModeToggleButton.setOnClickListener(v-> {
            if( !binding.triggerModeToggleButton.isChecked() ){
                AsReader.getInstance().getDeviceManager().setTriggerMode(DeviceConst.TriggerMode.MANUAL);
            } else {
                AsReader.getInstance().getDeviceManager().setTriggerMode(DeviceConst.TriggerMode.AUTO);
            }
        });

        binding.symbologySettingsButton.setOnClickListener(v -> NavHostFragment.findNavController(SettingFragment.this).navigate(R.id.action_SettingFragment_to_SymbologySettingsFragment));

        if(DeviceManager.getInstance().isConnected()) {
            Log.i(TAG_TEST, "Start getReaderSettings (onCreate)");
            DeviceManager.getInstance().getReaderSettings();
        }
        processGetInfo();
    }

    //-----------------------------------------------------------------------------
    //  Private method
    //-----------------------------------------------------------------------------
    private void processGetInfo() {
        switch (mProcessStep) {
            case UNPROCESSED:
                showProgressDialog();
                mProcessStep = GetProcessStep.SLEEP_TIME;
                AsReader.getInstance().getDeviceManager().getSleepTimeAndBeep();
                break;

            case SLEEP_TIME:
                mProcessStep = GetProcessStep.HID_MODE;
                AsReader.getInstance().getBarcodeManager().getBarcodeSettings(BarcodeConst.BarcodeSettings.CHARACTER_GET);
                break;

            case HID_MODE:
                mProcessStep = GetProcessStep.CHARACTER_STATUS;
                AsReader.getInstance().getDeviceManager().getHIDMode();
                break;

            case CHARACTER_STATUS:
                // Flow of standard
                mProcessStep = GetProcessStep.FINISH;
                requireActivity().runOnUiThread(this::hideProgressDialog);

                // Flow of NOTIS
//                mProcessStep = GetProcessStep.NOTIS_STATUS;
//                AsReader.getInstance().getBarcodeManager().setNotisEditing(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT, true);
//                AsReader.getInstance().getBarcodeManager().setNotisEditing(BarcodeConst.MemoryType.MEMORY_TYPE_TEMPORARY, true);
                break;

            case NOTIS_STATUS:
                // Need for flow of NOTIS
//                mProcessStep = GetProcessStep.FINISH;
//                requireActivity().runOnUiThread(this::hideProgressDialog);
                break;

            default:
                requireActivity().runOnUiThread(this::hideProgressDialog);
        }
    }

    private AlertDialog.Builder createCodeIdAlert() {

        return new AlertDialog.Builder(getActivity())
                .setTitle("Select Character CodeID Type")
                .setSingleChoiceItems(SettingConst.getCharacterStatusStringValues(), mCharacterStatus.ordinal(), (dialog, which) -> mCharacterStatus = SettingConst.getCharacterStatusValueFromInteger(which))
                .setPositiveButton("OK", (dialog, which) -> {
                    AsReader.getInstance().getBarcodeManager().setBarcodeSettings(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT, mCharacterStatus.getSettingsCode());
//                    SharedPreferencesHelper.getInstance().setCharacterSettings(mCharacterStatus.getSettingsCode());
                }).setNegativeButton("Cancel", null);
    }

    private AlertDialog.Builder createSleepAlert() {
        return new AlertDialog.Builder(getActivity())
                .setTitle("Select Sleep Time")
                .setSingleChoiceItems(SettingConst.getIntoSleepModeStringValues(), mIntoSleepMode.ordinal(), (dialog, which) -> mIntoSleepMode = SettingConst.IntoSleepMode.values()[which])
                .setPositiveButton("OK", (dialog, which) -> {
                    AsReader.getInstance().getDeviceManager().setSleepTime(mIntoSleepMode.getTime());
                }).setNegativeButton("Cancel", null);
    }

    private AlertDialog.Builder createApplicationBeepAlert() {
        mApplicationBeepSound = SharedPreferencesHelper.getInstance().getApplicationBeep();
        return new AlertDialog.Builder(getActivity())
                .setTitle("Select Application Beep")
                .setSingleChoiceItems(SettingConst.getApplicationBeepStringValues(), mApplicationBeepSound.ordinal(), (dialog, which) -> mApplicationBeepSound = SettingConst.ApplicationBeep.values()[which])
                .setPositiveButton("OK", (dialog, which) -> {
                    SharedPreferencesHelper.getInstance().setApplicationBeep(mApplicationBeepSound);
                    if( mApplicationBeepSound!=SettingConst.ApplicationBeep.NONE ) {
                        mMediaPlayer = getMediaPlayer(mApplicationBeepSound);
                        assert mMediaPlayer != null;
                        mMediaPlayer.start();
                    }
                }).setNegativeButton("Cancel", null);
    }

    private AlertDialog.Builder createOCRAlert() {
        final boolean[] checkedItems = SettingConst.getOCREnabled(mOCRInfo);
        return new AlertDialog.Builder(getActivity())
                .setTitle("Enable/Disable OCR")
                .setMultiChoiceItems(SettingConst.getOCRStringValues(), checkedItems, (dialog, which, isChecked) -> {
                    checkedItems[which] = isChecked;
                })
                .setPositiveButton("OK", (dialog, which) -> {
                    for (int i = 0; i < checkedItems.length; i++)
                        mOCRInfo.get(i).isEnableStatus = checkedItems[i];
                    AsReader.getInstance().getBarcodeManager().setOCRSettings(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT,mOCRInfo);
                }).setNegativeButton("Cancel", null);
    }

    private void changeSetting() {
        boolean beep = binding.beepToggleButton.isChecked();
        boolean vib = binding.vibrationToggleButton.isChecked();
        boolean aimer = binding.aimerToggleButton.isChecked();
        boolean led = binding.ledToggleButton.isChecked();
        boolean powerBeep = binding.powerBeepToggleButton.isChecked();

        AsReader.getInstance().getDeviceManager().setReaderSettings(beep, vib, aimer, led, powerBeep);

        SharedPreferencesHelper.getInstance().setBeep(beep);
        SharedPreferencesHelper.getInstance().setVibration(vib);
        SharedPreferencesHelper.getInstance().setAimer(aimer);
        SharedPreferencesHelper.getInstance().setLed(led);
        SharedPreferencesHelper.getInstance().setPowerBeep(powerBeep);
    }

    private void changeSleepSetting() {
        boolean sleepBeep = binding.sleepBeepToggleButton.isChecked();
        AsReader.getInstance().getDeviceManager().setSleepBeep(sleepBeep);
        SharedPreferencesHelper.getInstance().setSleepBeep(sleepBeep);
    }

    private MediaPlayer getMediaPlayer(SettingConst.ApplicationBeep applicationBeep){

        int soundID = applicationBeep.getSoundID();
        if( soundID!=0 ) {
            // rawにファイルがある場合
            MediaPlayer mediaPlayer = MediaPlayer.create(requireActivity().getApplicationContext(), soundID);
            // 音量調整を端末のボタンに任せる
            requireActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);
            return mediaPlayer;
        }
        return null;
    }

    //-----------------------------------------------------------------------------
    //  Public method
    //-----------------------------------------------------------------------------
    public void setChargeControlSuccess() {
        showToast("Success");
    }

    public void onSleepSettingReceived(Map<String, String> paramMap){
        Log.i(TAG_TEST, "!!!!!!!! onSleepTimeReceived !!!!!!!!");
        if(paramMap.containsKey(DeviceConst.KEY_PARAMETER_SLEEP_TIME)) {
            int sleepTime = Integer.parseInt(paramMap.get(DeviceConst.KEY_PARAMETER_SLEEP_TIME));
            this.mIntoSleepMode = SettingConst.getIntoSleepModeValueFromInteger(sleepTime);
        }
        if(paramMap.containsKey(DeviceConst.KEY_PARAMETER_SLEEP_BEEP)){
            boolean isSleepBeepOn = false;
            if(paramMap.get(DeviceConst.KEY_PARAMETER_SLEEP_BEEP).equals("true")) isSleepBeepOn = true;
            if(paramMap.get(DeviceConst.KEY_PARAMETER_SLEEP_BEEP).equals("false")) isSleepBeepOn = false;
            SharedPreferencesHelper.getInstance().setSleepBeep(isSleepBeepOn);
        }
        processGetInfo();
    }

    public void onHIDModeReceived(DeviceConst.HIDType hidType){
        if(hidType == DeviceConst.HIDType.NOT_APPLICABLE){
            binding.setHidToggleButton.setChecked(false);
            binding.setHidToggleButton.setEnabled(false);
        } else {
            boolean isHIDModeOn = false;
            if(hidType == DeviceConst.HIDType.HID_ON) isHIDModeOn = true;

            SharedPreferencesHelper.getInstance().setHidMode(isHIDModeOn);
            binding.setHidToggleButton.setChecked(isHIDModeOn);
        }
        if(mProcessStep!=GetProcessStep.FINISH)
            processGetInfo();
    }

    public void getReaderSettings(Map<String, Boolean> settingParamMap){
        binding.beepToggleButton.setChecked(settingParamMap.get(DeviceConst.KEY_PARAMETER_SCAN_BEEP));
        binding.vibrationToggleButton.setChecked(settingParamMap.get(DeviceConst.KEY_PARAMETER_SCAN_VIBRATION));
        binding.aimerToggleButton.setChecked(settingParamMap.get(DeviceConst.KEY_PARAMETER_SCAN_AIMER));
        binding.ledToggleButton.setChecked(settingParamMap.get(DeviceConst.KEY_PARAMETER_LED));
        binding.powerBeepToggleButton.setChecked(settingParamMap.get(DeviceConst.KEY_PARAMETER_POWER_BEEP));
    }

    //-----------------------------------------------------------------------------
    //  BaseFragment Override method
    //-----------------------------------------------------------------------------
    @Override
    public void onSetReaderSettingSuccess() {
        hideProgressDialog();
        showToast("Success");
    }

    @Override
    public void onConnect(boolean isConnect) {
        Log.d(TAG, "onConnect");

        if( isConnect ) {
            Log.i(TAG_TEST, "Start getReaderSettings(onConnect)");
            DeviceManager.getInstance().getReaderSettings();
        } else {
            NavHostFragment.findNavController(SettingFragment.this).navigate(R.id.action_SettingFragment_to_MainFragment);
        }
    }

    @Override
    public void receivedAppStatus(MainActivity.AppStatus status) {
        if( status==MainActivity.AppStatus.BACKGROUND || status==MainActivity.AppStatus.INTO_BACKGROUND) {
            NavHostFragment.findNavController(SettingFragment.this).navigate(R.id.action_SettingFragment_to_MainFragment);
        }
    }

    @Override
    public void onDeviceError(String message) {
        Log.d(TAG, "Error message : " + message);
        hideProgressDialog();
        showToast("error : " + message);
    }

    //-----------------------------------------------------------------------------
    //  BarcodeManagerCallback
    //-----------------------------------------------------------------------------
    @Override
    public void onReceivedResponse(BarcodeConst.BarcodeCommandAction type, BarcodeResult result) {

        Log.d(TAG_TEST, "onReceivedResponse : type = " + type + " Status = " + result.getStatus());

        if( result.getStatus()==BarcodeResult.Status.FAILED ) { return; }

        switch (type) {
            case GET_INFO_CHARACTER_STATUS:
            case SETTINGS_CHARACTER_STATUS:
                Log.d(TAG_CHARACTER_TEST, "onReceivedResponse : type = " + type + " Status = " + result.getStatus());
                requireActivity().runOnUiThread(() -> {
                    switch (result.getResult()) {
                        case CHARACTER_NONE:
                            mCharacterStatus = SettingConst.CharacterStatus.NONE;
                            break;
                        case CHARACTER_SYMBOL:
                            mCharacterStatus = SettingConst.CharacterStatus.SYMBOL;
                            break;
                        case CHARACTER_AIM:
                            mCharacterStatus = SettingConst.CharacterStatus.AIM;
                            break;
                    }
                });
                break;

            case GET_INFO_STOP_CONDITION:
                Log.i(TAG_TEST, "StopOptionMap : " + result.getMessage());
                break;

            case GET_INFO_OCR_ENABLE_STATUS:
                mOCRInfo = result.getOCRInfoArray();
                break;

            case SETTINGS_NOTIS_EDITING:
            case SETTING_OCR:
            case GET_INFO_SYSTEM_BEEP:
                break;
        }
        processGetInfo();
    }

    @Override
    public void onBarcodeManagerError(BarcodeManagerError error) {

    }
}