package jp.co.asterisk.asreader.a24d.demoapp.fragments;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.navigation.fragment.NavHostFragment;
import androidx.recyclerview.widget.LinearLayoutManager;

import java.util.ArrayList;
import java.util.Collections;

import jp.co.asterisk.asreader.a24d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.a24d.demoapp.databinding.FragmentSymbologySettingsBinding;
import jp.co.asterisk.asreader.a24d.demoapp.util.symbology.SymbologyPreferencesHelper;
import jp.co.asterisk.asreader.a24d.demoapp.util.symbology.SymbologyRowData;
import jp.co.asterisk.asreader.a24d.demoapp.util.symbology.SymbologySettingsRecycleViewAdapter;
import jp.co.asterisk.asreader.a24d.demoapp.util.symbology.SymbologySettingsRecycleViewAdapterCallback;
import jp.co.asterisk.asreader.a24d.sdk.AsReader;
import jp.co.asterisk.asreader.barcodemanager.BarcodeManagerCallback;
import jp.co.asterisk.asreader.barcodemanager.SymbologyInfoModel;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeConst;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeManagerError;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeResult;
import jp.co.asterisk.asreader.barcodemanager.define.Symbology;
import jp.co.asterisk.asreader.devicemanager.DeviceConst;
import jp.co.asterisk.asreader.devicemanager.DeviceManager;
import jp.co.asterisk.asreader.devicemanager.DeviceManagerCallback;

public class SymbologySettingsFragment extends BaseFragment implements BarcodeManagerCallback, DeviceManagerCallback, SymbologySettingsRecycleViewAdapterCallback {
    private static final String TAG = "【AsReader DemoApp】" + SymbologySettingsFragment.class.getSimpleName();
    private static final String TAG_TEST = "【A24DDemoApp SettingsFragmentTest】" + SymbologySettingsFragment.class.getSimpleName();
    private SymbologySettingsRecycleViewAdapter mSymbologySettingsViewAdapter = null;

    private FragmentSymbologySettingsBinding binding;

    private ArrayList<SymbologyRowData> mRowDataList = null;

    private boolean isSettingSymbolAll = false;
    private boolean isSettingSymbolAllStatus = false;

    //-----------------------------------------------------------------------------
    //  Constructor
    //-----------------------------------------------------------------------------
    public SymbologySettingsFragment() {
        // Required empty public constructor
    }

    //-----------------------------------------------------------------------------
    //  Lifecycle
    //-----------------------------------------------------------------------------
    @Override
    public void onCreate(Bundle savedInstanceState) {
        AsReader.getInstance().getBarcodeManager().setCallback(this);
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        binding = FragmentSymbologySettingsBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onStart(){
        super.onStart();
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setTitle(getString(R.string.symbology_setting));
        setBackButtonVisibility(View.VISIBLE);
        setRightButtonVisibility(View.INVISIBLE);

        Button mSaveButton = (Button)view.findViewById(R.id.save_symbology_settings_button);
        mSaveButton.setOnClickListener(v->{
            Log.d(TAG_TEST, "======== onClick SaveButton ========");
            ArrayList<SymbologyInfoModel> sendList = new ArrayList<>();
            for( SymbologyRowData rowData : mRowDataList) {
                SymbologyInfoModel model = new SymbologyInfoModel(rowData.getSymbology(), rowData.getSymbologyEnable());
                Log.d(TAG_TEST, "Symbol = " + model.symbology + "  Enable = " + (model.isEnableStatus?"TRUE":"FALSE"));
                sendList.add(model);
            }
            SymbologyPreferencesHelper.getInstance().setPreferenceSymbologiesArray(sendList);
            AsReader.getInstance().getBarcodeManager().setSymbologySettings(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT, sendList);
        });

        Button symbolEnableButton = (Button)view.findViewById(R.id.symbol_enable_all_button);
        symbolEnableButton.setOnClickListener(v->{
            isSettingSymbolAll = true;
            isSettingSymbolAllStatus = true;
            AsReader.getInstance().getBarcodeManager().setSymbologyAllEnable(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT, isSettingSymbolAllStatus);
        });

        Button symbolDisableButton = (Button)view.findViewById(R.id.symbol_disable_all_button);
        symbolDisableButton.setOnClickListener(v-> {
            isSettingSymbolAll = true;
            isSettingSymbolAllStatus = false;
            AsReader.getInstance().getBarcodeManager().setSymbologyAllEnable(BarcodeConst.MemoryType.MEMORY_TYPE_PERMANENT, isSettingSymbolAllStatus);
        });

        if( DeviceManager.getInstance().isConnected() ) {
            AsReader.getInstance().getBarcodeManager().getStopCondition();

            ArrayList<Symbology> symbologies = new ArrayList<>();
            Collections.addAll(symbologies, Symbology.values());
            AsReader.getInstance().getBarcodeManager().getSymbologySettings(symbologies);
        }
    }

    //-----------------------------------------------------------------------------
    //  Private method
    //-----------------------------------------------------------------------------
    private void setUpRecyclerView(ArrayList<SymbologyInfoModel> updateArrayList) {

        if( updateArrayList==null ){ return; }
        boolean isFirst = false;
        if( mRowDataList==null ) {
            // First create
            mRowDataList = new ArrayList<>();
            isFirst = true;
        }

        for( int i=0; i<updateArrayList.size(); i++ ) {
            SymbologyInfoModel model = updateArrayList.get(i);

            SymbologyRowData row = new SymbologyRowData();
            row.setTitle(model.symbology.getString());
            row.setSymbology(model.symbology);
            row.setSymbologyEnable(model.isEnableStatus);

            if (isFirst) {
                mRowDataList.add(row);
            } else {
                if ( model.symbology== mRowDataList.get(i).getSymbology() &&
                        model.isEnableStatus!=mRowDataList.get(i).getSymbologyEnable() ) {
                    mRowDataList.get(i).setSymbologyEnable(model.isEnableStatus);
                }
            }
        }

        if (isFirst) {
            mSymbologySettingsViewAdapter = new SymbologySettingsRecycleViewAdapter(mRowDataList);
            mSymbologySettingsViewAdapter.setCallback(SymbologySettingsFragment.this);

            LinearLayoutManager llm = new LinearLayoutManager(SymbologySettingsFragment.this.getContext());
            binding.symbologySettingsRecyclerView.setHasFixedSize(true);
            binding.symbologySettingsRecyclerView.setLayoutManager(llm);
            binding.symbologySettingsRecyclerView.setAdapter(mSymbologySettingsViewAdapter);

            mSymbologySettingsViewAdapter.setEventEnable(true);

        } else {
            mSymbologySettingsViewAdapter.notifyDataSetChanged();
        }

        SymbologyPreferencesHelper.getInstance().setPreferenceSymbologiesArray(updateArrayList);
    }

    private void updateData( ArrayList<SymbologyInfoModel> symbologies ) {
        for(int i=0; i<mRowDataList.size(); i++) {
            if( mRowDataList.get(i).getSymbology()==symbologies.get(i).symbology ){
                Log.d(TAG_TEST, "Symbol = " + symbologies.get(i).symbology + " Status = " + symbologies.get(i).isEnableStatus);
                if( mRowDataList.get(i).getSymbologyEnable()!=symbologies.get(i).isEnableStatus ) {
                    mRowDataList.get(i).setSymbologyEnable(symbologies.get(i).isEnableStatus);
                }
            }
        }
        mSymbologySettingsViewAdapter.notifyDataSetChanged();
    }

    //-----------------------------------------------------------------------------
    //  DeviceManager Callback
    //-----------------------------------------------------------------------------
    @Override
    public void onConnect(boolean isConnect) {
        Log.d(TAG, "onConnect");
        if(!isConnect){
            NavHostFragment.findNavController(SymbologySettingsFragment.this).navigate(R.id.action_SymbologySettingsFragment_to_MainFragment);
        }
    }

    @Override
    public void receivedAppStatus(MainActivity.AppStatus status) {
        if( status==MainActivity.AppStatus.BACKGROUND || status==MainActivity.AppStatus.INTO_BACKGROUND) {
            NavHostFragment.findNavController(SymbologySettingsFragment.this).navigate(R.id.action_SymbologySettingsFragment_to_MainFragment);
        }
    }

    @Override
    public void onBatteryStateReceived(int battery) {

    }

    @Override
    public void onTriggerEventReceived(DeviceConst.DeviceTriggerEvent event) {

    }

    @Override
    public void onDeviceManagerError(jp.co.asterisk.asreader.utility.AsReaderError.ErrorCode code) {

    }

    //-----------------------------------------------------------------------------
    //  BarcodeManager Callback
    //-----------------------------------------------------------------------------
    @Override
    public void onReceivedResponse(BarcodeConst.BarcodeCommandAction type, BarcodeResult result) {
        requireActivity().runOnUiThread(() -> {
            switch (type) {
                case SETTINGS_SYMBOLOGIES:
                    if(isSettingSymbolAll) {
                        isSettingSymbolAll = false;
                    }
                    Log.d(TAG_TEST, "onReceivedSymbologySettingsInfo Update!!!!");
                    updateData( result.getSymbologyInfoArray() );
                    SymbologyPreferencesHelper.getInstance().setPreferenceSymbologiesArray(result.getSymbologyInfoArray());
                    break;

                case GET_INFO_SYMBOLOGIES_ENABLE_STATUS:
                    Log.d(TAG_TEST, "onReceivedSymbologyGetInfo Update!!!!");
                    setUpRecyclerView(result.getSymbologyInfoArray());
                    updateData( result.getSymbologyInfoArray() );
                    break;
            }
        });
    }

    @Override
    public void onBarcodeManagerError(BarcodeManagerError error) {

    }

    //-----------------------------------------------------------------------------
    //  SymbologySettingsRecycleViewAdapter Callback
    //-----------------------------------------------------------------------------
    @Override
    public void onChangedSwitchCheckStatus(boolean isChecked, SymbologyRowData data) {

        int index = data.getSymbology().ordinal();

        for(int i=0; i<mRowDataList.size(); i++) {
            if( mRowDataList.get(i).getSymbology()==data.getSymbology() ){
                mRowDataList.get(index).setSymbologyEnable(isChecked);
                break;
            }
        }

        Log.d(TAG_TEST, "onChangedSwitchCheckStatus isChecked = " + (isChecked?"True":"False") + "  Symbology = " + data.getSymbology());
    }
}
