package jp.co.asterisk.asreader.a24d.demoapp.util;

import java.util.ArrayList;

import jp.co.asterisk.asreader.a24d.demoapp.R;
import jp.co.asterisk.asreader.barcodemanager.define.BarcodeConst;
import jp.co.asterisk.asreader.barcodemanager.define.OCR;
import jp.co.asterisk.asreader.barcodemanager.model.OCRInfoModel;

public class SettingConst {

    public static int batteryValue = 0;

    public enum CharacterStatus {
        NONE {
            @Override
            public BarcodeConst.BarcodeSettings getSettingsCode() { return BarcodeConst.BarcodeSettings.CHARACTER_NONE; }
            @Override
            public String getText() { return "NONE"; }
        },
        SYMBOL {
            @Override
            public BarcodeConst.BarcodeSettings getSettingsCode() { return BarcodeConst.BarcodeSettings.CHARACTER_CODE_ID_SYMBOL; }
            @Override
            public String getText() { return "SYMBOL"; }
        },
        AIM {
            @Override
            public BarcodeConst.BarcodeSettings getSettingsCode() { return BarcodeConst.BarcodeSettings.CHARACTER_CODE_ID_AIM; }
            @Override
            public String getText() { return "AIM"; }
        };
        abstract public BarcodeConst.BarcodeSettings getSettingsCode();
        abstract public String getText();
    }

    public enum IntoSleepMode {
        MINUTE_5 {
            @Override
            public int getTime() { return 5; }
            @Override
            public String getText() { return "5 min"; }
        },
        MINUTE_10 {
            @Override
            public int getTime() { return 10; }
            @Override
            public String getText() { return "10 min"; }
        },
        MINUTE_30 {
            @Override
            public int getTime() { return 30; }
            @Override
            public String getText() { return "30 min"; }
        },
        MINUTE_60 {
            @Override
            public int getTime() { return 60; }
            @Override
            public String getText() { return "60 min"; }
        },
        MINUTE_120 {
            @Override
            public int getTime() { return 120; }
            @Override
            public String getText() { return "120 min"; }
        },
        NONE {
            @Override
            public int getTime() { return 0; }
            @Override
            public String getText() { return "Non Sleep"; }
        };

        abstract public int getTime();
        abstract public String getText();
    }

    public enum ApplicationBeep {
        NONE {
            @Override
            public String getFileName() { return "None"; }
            @Override
            public String getText() { return "None"; }
            @Override
            public int getSoundID() { return 0;}
        },
        BEEP1 {
            @Override
            public String getFileName() { return "beep1.mp3"; }
            @Override
            public String getText() { return "Beep1"; }
            @Override
            public int getSoundID() { return R.raw.beep1;}
        },
        BEEP2 {
            @Override
            public String getFileName() { return "beep2.mp3"; }
            @Override
            public String getText() { return "Beep2"; }
            @Override
            public int getSoundID() { return R.raw.beep2;}
        },
        BOYON {
            @Override
            public String getFileName() { return "boyon.mp3"; }
            @Override
            public String getText() { return "Boyon"; }
            @Override
            public int getSoundID() { return R.raw.boyon;}
        },
        JAPANESE_DRUM1 {
            @Override
            public String getFileName() { return "japanese_drum1.mp3"; }
            @Override
            public String getText() { return "Japanese Drum1"; }
            @Override
            public int getSoundID() { return R.raw.japanese_drum1;}
        },
        JAPANESE_DRUM2 {
            @Override
            public String getFileName() { return "japanese_drum2.mp3"; }
            @Override
            public String getText() { return "Japanese Drum2"; }
            @Override
            public int getSoundID() { return R.raw.japanese_drum2;}
        },
        TRUMPET {
            @Override
            public String getFileName() { return "trumpet.mp3"; }
            @Override
            public String getText() { return "Trumpet"; }
            @Override
            public int getSoundID() { return R.raw.trumpet;}
        },
        PING1 {
            @Override
            public String getFileName() { return "ping1.mp3"; }
            @Override
            public String getText() { return "Ping1"; }
            @Override
            public int getSoundID() { return R.raw.ping1;}
        },
        PING2 {
            @Override
            public String getFileName() { return "ping2.mp3"; }
            @Override
            public String getText() { return "Ping2"; }
            @Override
            public int getSoundID() { return R.raw.ping2;}
        },
        PING3 {
            @Override
            public String getFileName() { return "ping3.mp3"; }
            @Override
            public String getText() { return "Ping3"; }
            @Override
            public int getSoundID() { return R.raw.ping3;}
        },
        CHILDREN {
            @Override
            public String getFileName() { return "children.wav"; }
            @Override
            public String getText() { return "Children"; }
            @Override
            public int getSoundID() { return R.raw.children;}
        };

        abstract public String getFileName();
        abstract public String getText();
        abstract public int getSoundID();
    }

    public enum OCRType {
        OCR_A {
            @Override
            public OCR getOCR() { return OCR.OCR_A; }
            @Override
            public String getText() { return "OCR A"; }
        },
        OCR_B {
            @Override
            public OCR getOCR() { return OCR.OCR_B; }
            @Override
            public String getText() { return "OCR B"; }
        };
        abstract public OCR getOCR();
        abstract public String getText();
    }

    static public String[] getCharacterStatusStringValues() {
        String[] values = new String[CharacterStatus.values().length];

        for( int i=0; i<CharacterStatus.values().length; i++ ) {
            CharacterStatus[] status = CharacterStatus.values();
            values[i] = status[i].getText();
        }
        return values;
    }

    static public CharacterStatus getCharacterStatusValueFromInteger(int val) {
        for( CharacterStatus status : CharacterStatus.values() ) {
            if(status.ordinal()==val) {
                return status;
            }
        }
        return CharacterStatus.NONE;
    }

    static public String[] getIntoSleepModeStringValues() {
        String[] values = new String[IntoSleepMode.values().length];

        for( int i=0; i<IntoSleepMode.values().length; i++ ) {
            IntoSleepMode[] modes = IntoSleepMode.values();
            values[i] = modes[i].getText();
        }
        return values;
    }

    static public String[] getApplicationBeepStringValues() {
        String[] values = new String[ApplicationBeep.values().length];

        for( int i=0; i<ApplicationBeep.values().length; i++ ) {
            ApplicationBeep[] modes = ApplicationBeep.values();
            values[i] = modes[i].getText();
        }
        return values;
    }

    static public IntoSleepMode getIntoSleepModeValueFromInteger(int val) {
        for( IntoSleepMode mode : IntoSleepMode.values() ) {
            if(mode.getTime()==val) {
                return mode;
            }
        }
        return IntoSleepMode.MINUTE_5;
    }

    static public ArrayList<OCRInfoModel> getFalseOCRArray(){
        ArrayList<OCRInfoModel> ocrInfoModelArrayList = new ArrayList<>();

        for(OCR ocr : OCR.values() ){
            ocrInfoModelArrayList.add(new OCRInfoModel(ocr,false));
        }

        return ocrInfoModelArrayList;   // This is just OCR enum array, isEnableStatus value in it is ALL FALSE!!!!
    }

    static public String[] getOCRStringValues(){
        String[] values = new String[OCRType.values().length];

        for( int i=0; i<OCRType.values().length; i++ ) {
            OCRType[] ocrs = OCRType.values();
            values[i] = ocrs[i].getText();
        }
        return values;
    }

    static public boolean[] getOCREnabled(ArrayList<OCRInfoModel> ocrInfoModelArrayList){
        boolean[] values = new boolean[ocrInfoModelArrayList.size()];

        for ( int i=0; i<ocrInfoModelArrayList.size(); i++) {
            values[i] = ocrInfoModelArrayList.get(i).isEnableStatus;
        }
        return values;
    }

}
