package jp.co.asterisk.asreader.a24d.demoapp.util.symbology;

import android.content.Context;
import android.content.SharedPreferences;

import java.util.ArrayList;

import jp.co.asterisk.asreader.barcodemanager.define.Symbology;
import jp.co.asterisk.asreader.barcodemanager.SymbologyInfoModel;

// This class saves symbology settings in preference.
// Because preference can only save string data, this class converts symbology list to string data and vice versa.
public class SymbologyPreferencesHelper {

    private static final String TAG = "【A24DDemoApp】" + SymbologyPreferencesHelper.class.getSimpleName();
    private static final String SETTINGS_SYMBOLOGY = "settings_symbology";

    private final String SYMBOLOGY_TRUE = "True";
    private final String SYMBOLOGY_FALSE = "False";

    private static SymbologyPreferencesHelper instance;

    private Context mContext = null;

    //-----------------------------------------------------------------------------
    //  Constructor
    //-----------------------------------------------------------------------------
    private SymbologyPreferencesHelper() {
    }

    //-----------------------------------------------------------------------------
    //  Public method
    //-----------------------------------------------------------------------------
    public static SymbologyPreferencesHelper getInstance(){
        if (instance==null) {
            instance = new SymbologyPreferencesHelper();
        }
        return instance;
    }

    public void initialize(Context context) {
        mContext = context;

        // Initialize
        Symbology[] symblogyValues = Symbology.values();

        for (Symbology symbology : symblogyValues) {
            String symbolEnableStr = getSettingItem(symbology.getString());

            if (symbolEnableStr == null) {

                switch (symbology) {
                    case UPC_A:
                    case UPC_E:
                    case EAN8_JAN8:
                    case EAN13_JAN13:
                    case CODE_39:
                    case CODA_BAR:
                    case QR_CODE:
                    case DATA_MATRIX:
                    case CODE_128:
                        putSettingItem(symbology.getString(), SYMBOLOGY_TRUE);
                        break;

                    default:
                        putSettingItem(symbology.getString(), SYMBOLOGY_FALSE);

                }
            }
        }
    }

    public void setPreferenceSymbologiesArray(ArrayList<SymbologyInfoModel> symbologies) {

        for( SymbologyInfoModel symbolInfo : symbologies ){
            setPreferenceSymbology(symbolInfo);
        }
    }

    public void setPreferenceSymbology(SymbologyInfoModel symbologyInfo) {
        String enableStr = getSettingItem(symbologyInfo.symbology.getString());
        boolean enable = false;

        if( enableStr!=null ) {
            if( enableStr.contentEquals(SYMBOLOGY_TRUE) ) {
                enable = true;
            }
        }

        if( enable!=symbologyInfo.isEnableStatus ) {
            putSettingItem(symbologyInfo.symbology.getString(), (symbologyInfo.isEnableStatus?SYMBOLOGY_TRUE:SYMBOLOGY_FALSE) );
        }
    }

    public ArrayList<SymbologyInfoModel> getPreferenceSymbologiesArray() {
        Symbology[] symblogyValues = Symbology.values();
        ArrayList<SymbologyInfoModel> retArrayList = new ArrayList<>();

        for( Symbology symbology : symblogyValues ){
            SymbologyInfoModel model = getPreferenceSymbology(symbology);
            if( model!=null ) {
                retArrayList.add(model);
            }
        }
        return retArrayList;
    }

    public SymbologyInfoModel getPreferenceSymbology(Symbology symbology) {
        String symbolEnableStr = getSettingItem(symbology.getString());

        if( symbolEnableStr!=null ) {
            boolean enable = false;
            if( symbolEnableStr.contentEquals(SYMBOLOGY_TRUE) ) { enable = true; }

            return new SymbologyInfoModel(symbology, enable);
        }
        return null;
    }

    //-----------------------------------------------------------------------------
    //  Private method
    //-----------------------------------------------------------------------------

    private void putSettingItem(String key, String value) {
//        Log.d(TAG, "Put " + key +" (value : " + value + " ) to " + SETTINGS_SYMBOLOGY);

        SharedPreferences preferences = mContext.getSharedPreferences(SETTINGS_SYMBOLOGY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putString(key, value);
        editor.apply();
    }

    private String getSettingItem(String key) {
//        Log.d(TAG, "Get " + key + " from " + SETTINGS_SYMBOLOGY);
        return mContext.getSharedPreferences(SETTINGS_SYMBOLOGY, 0).getString(key, null);
    }
}
